#include <def21161.h>
#include <asm_sprt.h>

#include "UART_Driver.h"

.section /dm seg_dmda;
.var	_UART_Error = 0;
.global _UART_Error;

/***************************************************************************************

	EZ-UART/MIDI Driver
	  
 	  written by Dan Ledger
	  last modified 12/2002
	

	Rev 1.4

	    Revision History

		1.0 - original driver
		1.1 - added push sts to interrupt routines
		1.2 - added automatic setup of interrupt vectors
		1.3 - changed length of receive SPORT to 29
		1.4	- corrected bug in UARTTX_Service - as this routine is no longer called from 
			  C but rather is automatically called from the ISR, the return should be an
			  RTI, not leaf_exit;
				
****************************************************************************************/



/***************************************************************************************
	Setup_UART
		
	Input:
		r4 = DSP operating frequency
		r8 = desired BAUD rate
		
	Output:
		f0 = divisor error.  this is the fractional portion resulting from the truncation
							 to fixed point of the sport clock divisor

	C Prototype:
		float	Setup_UART( float DSP_Frequency, float BAUD );

****************************************************************************************/
.section /pm seg_pmco;
_Setup_UART:
.global _Setup_UART;

		r0=0x00000000;				// initially clear SPORT control register
		dm(SPCTL1)=r0;          // tx on sport 1
   		dm(SPCTL3)=r0;          // rx on sport 3
		dm(SP13MCTL) = R0;
		ustat1=r0;
	
		// Setup SPORT1 - Transmit
		bit set ustat1 DDIR | SENDN | LAFS | LFS | IFS | FSR | ICLK | SLEN11 | SPEN_A;
		dm(SPCTL1) = ustat1;

		// Setup SPORT3 - Receive
		ustat1 = 0;
		bit set ustat1 LAFS | LFS | FSR | ICLK | SLEN29 | SPEN_B | CKRE;
		dm(SPCTL3) = ustat1;

		// Calculate divisors based on DSP clock frequency and BAUD rate		
		r2=1086324736;
		F2=F2*F8;		
		F1=recips F2;
		r8=1073741824;
		F12=F1*F2;
		F4=F1*F4, F2=F8-F12;
		F12=F2*F12;
		F4=F2*F4, F2=F8-F12;
		F12=F2*F12;
		F4=F2*F4, F2=F8-F12;
		r8=pass r2;
		F2=F2*F4, F12=F8-F12;
		F12=F12*F2;
		F0=F12+F2;
		
		R4 = FIX F0;
		F8 = FLOAT R4;
		F0 = F0-F8;

		R8 = 3;
		R12 = R4 * R8 (uui);
		// Setup Transmit
		R1 = 11<<16;		// 32 bits per frame 
		R2 = R1 + R12;
		dm(DIV1) = R2;

		// Setup Receive
		// To enable MIDI RX with UART transmit, hardwire RX clock to 31250

#ifdef	MIDI_RX		
		R4 = 533;
#endif		
		dm(DIV3) = R4;
		// setup SPORT1 and SPORT3 ISRs

		// UART Transmit
		r0 = _UartTx_Service;
		r1 = lshift r0 by 16;
		px1 = r1;
		r1 = b#000001100011111;
		r0 = lshift r0 by -16;
		r1 = lshift r1 by 17;
		r1 = r1 OR r0;
		px2 = r1;
		pm(0x4002C) = px;
		px2 = 0x0b3e0000;
		px1 = 0;
		pm(0x4002D) = px;
		pm(0x4002E) = px;
		pm(0x4002F) = px;

		bit set imask SP1I | SP3I;
		leaf_exit;
_Setup_UART.end:
									

/***************************************************************************************
	UartTX_Service
		
	Description:
		This function handles the TX interrupt and moving data from the UART_Transmit_Buffer
		out to the serial port.  This function should be placed in the SPORT 1 ISR.

****************************************************************************************/

.section /dm seg_dmda;
.var	UART_Transmit_Buffer[UART_TX_BUFFER_LEN];
.var	UART_Transmit_Buffer_Read_Ptr=0;		
.var	UART_Transmit_Buffer_Write_Ptr=0;		

.section /pm seg_pmco;
_UartTx_Service:
.global _UartTx_Service;

	push sts;

	// * BACKUP VARIABLES 
	puts = r0;
	puts = r1;
	// * END BACKUP VARIABLES 

	r0 = dm(UART_Transmit_Buffer_Read_Ptr);
	r1 = dm(UART_Transmit_Buffer_Write_Ptr);
	comp(r0,r1);
	if eq jump UART_Nothing_To_Transmit;

	// BACKUP MORE VARIABLES
	r1 = i4;	puts = r1;
	r1 = m4;	puts = r1;
	puts = r4;
		
	i4 = UART_Transmit_Buffer;
	m4 = dm(UART_Transmit_Buffer_Read_Ptr);
	r4 = dm(m4,i4);
	
	// increment pointer
	r0 = m4;	r0 = r0 + 1;
	
	// See if pointer has wrapped
	r1 = UART_TX_BUFFER_LEN;
	comp(r1,r0);
	if EQ r0 = r0 - r1;
	dm(UART_Transmit_Buffer_Read_Ptr) = r0;

	dm(TX1A) = r4;

	// * RESTORE VARIABLES 
	r4 = gets(1);
	r1 = gets(2);	m4 = r1;
	r1 = gets(3); 	i4 = r1;
	r1 = gets(4);
	r0 = gets(5);
	alter(5);
	pop sts;
	rti;
	// * END RESTORE VARIABLES 

	
// If there is nothing more to transmit, disable the transmit interrupt	
UART_Nothing_To_Transmit:
	
	bit clr IMASK SP1I;
	// * RESTORE VARIABLES 
	r1 = gets(1);
	r0 = gets(2);
	alter(2);
	pop sts;
	rti;
	// * END RESTORE VARIABLES 

_UartTx_Service.end:
														





/***************************************************************************************
	Uart_TX_Word
		
	Description:
		This function writes a value to the SPORT1 transmit register.  It first takes
		the 8-bit value and creates a 30-bit value for transmitting.
		

****************************************************************************************/
.section /pm seg_pmco;
_UartTx_Word:
.global _UartTx_Word;

	r0 = r4;

	r2 = b#01111111100;				
	r4 = b#10000000001;
	r1 = lshift r0 by 2;
	r1 = r1 and r2;
	r1 = r1 OR r4;
		
	r0 = dm(UART_Transmit_Buffer_Read_Ptr);
	r2 = dm(UART_Transmit_Buffer_Write_Ptr);
	r0 = r0 - 1;
	comp(r0,r2);
	if eq jump UART_TX_Full;
	
	i4 = UART_Transmit_Buffer;
	m4 = dm(UART_Transmit_Buffer_Write_Ptr);
	dm(m4,i4) = r1;
	
	// increment pointer
	r0 = m4;	r0 = r0 + 1;
	
	// See if pointer has wrapped
	r1 = UART_TX_BUFFER_LEN;
	comp(r1,r0);
	if EQ r0 = r0 - r1;
	dm(UART_Transmit_Buffer_Write_Ptr) = r0;
	
	// if the transmit interrupt had been disabled, re-enable it to transmit this new data
	bit clr IRPTL SP1I;
	bit set IMASK SP1I;

	leaf_exit;
_UartTx_Word.end:							

/***************************************************************************************
	UartTx_Buffer
		
	Description:
		This function writes a buffer of data to the SPORT1 transmit register.  It first takes
		the 8-bit value and creates a 30-bit value for transmitting.
		

****************************************************************************************/
.section /dm seg_dmda;
.var	UART_i3_store;


.section /pm seg_pmco;
_UartTx_Buffer:
.global _UartTx_Buffer;

	i4 = r4;
	LCNTR = r8, do (PC,UART_Transmit_Loop-1) until LCE;
		r4=dm(i4,m6);
		r0 = r4;
		r2 = b#01111111100;				// binary 111 (a 'one' at 3 x baud rate)
		r4 = b#10000000001;
		r1 = lshift r0 by 2;
		r1 = r1 and r2;
		r1 = r1 OR r4;
		
		r0 = dm(UART_Transmit_Buffer_Read_Ptr);
		r2 = dm(UART_Transmit_Buffer_Write_Ptr);
		r0 = r0 - 1;
		comp(r0,r2);
		if eq jump UART_TX_Full (la);
		
		dm(UART_i3_store) = i3;
		i3 = UART_Transmit_Buffer;
		m4 = dm(UART_Transmit_Buffer_Write_Ptr);
		dm(m4,i3) = r1;
		i3 = dm(UART_i3_store);
		
		// increment pointer
		r0 = m4;	r0 = r0 + 1;
		
		// See if pointer has wrapped
		r1 = UART_TX_BUFFER_LEN;
		comp(r1,r0);
		if EQ r0 = r0 - r1;
		dm(UART_Transmit_Buffer_Write_Ptr) = r0;
UART_Transmit_Loop:	


	// if the transmit interrupt had been disabled, re-enable it to transmit this new data
	bit clr IRPTL SP1I;
	bit set IMASK SP1I;
	leaf_exit;

UART_TX_Full:
	r0 = TX_BUFFER_OVERFLOW;
	dm(_UART_Error) = r0;
	bit set imask SFT3I;		// raise software interrupt 3 when UART error detected.
	bit clr IRPTL SP1I;
	bit set IMASK SP1I;
	leaf_exit;

_UartTx_Buffer.end:	
	

/***************************************************************************************
	Uart_Rx
		
	Description:
		Once a SPORT3 receive interrupt occurs, this function should be called (from C)
		to retrieve the and interpret the value.  This routine captures the input from RX3A,
		a derives the 8-bit value.
		
	int	UartRx();
		

****************************************************************************************/
.section /pm seg_pmco;
_UartRx:
.global _UartRx;

	// receive in a 30 bit transmission, generate rx interrupt
	//  start bit triggers frame sync (rfs and dr tied together
	r0 = dm(RX3B);
	
	r4 = fext r0 by 25:1;		// bit 0
	r1 = fext r0 by 22:1;		// bit 1
	r4 = r4 or fdep r1 by 1:1;

	r1 = fext r0 by 19:1;		// bit 2
	r4 = r4 or fdep r1 by 2:1;

	r1 = fext r0 by 16:1;		// bit 3
	r4 = r4 or fdep r1 by 3:1;

	r1 = fext r0 by 13:1;		// bit 4
	r4 = r4 or fdep r1 by 4:1;

	r1 = fext r0 by 10:1;		// bit 5
	r4 = r4 or fdep r1 by 5:1;

	r1 = fext r0 by 7:1;		// bit 6
	r4 = r4 or fdep r1 by 6:1;

	r1 = fext r0 by 4:1;		// bit 7
	r4 = r4 or fdep r1 by 7:1;

	// r4 contains received word
	r0 = r4;

	leaf_exit;
_UartRx.end:
 
